/*
 * @(#)WStack.cs        1.0 12/26/2007
 * 
 * Copyright (c) 2007 Senamirmir Project. All Rights Reserved.
 *
 * This file is part of Software: WashRa 4.0
 *
 * WashRa 4.0 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * WashRa is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace washra {
   /**
    * This is simply a generic stack used by the convertor program
    */
   class WStack<T> {
      // when the stack is empty, this value is returned
      Object returnType = -1;
      // the stack underlying storage
      T[] rack;
      // index used to point to the last element in the stack
      int top = -1;
      // stack initial size
      int size = 32;

      /**
       * Default constructor. Initial stack size is 32
       */
      public WStack() : this(32) { }

      /**
       * Constructor handles user size request
       */ 
      public WStack(int userSize) {
         size = (userSize >= 0 ? userSize : size);
         rack = new T[size];
      }

      /**
       * Stores the given element on top of the stack
       */ 
      public void push(T item) {
         if (!isFull()) {
            rack[++top] = item;
         } else {
            Console.WriteLine("Error: Stack is full");
         }
      }

      /**
       * Returns the element from the top the stack and adjust index
       */ 
      public T pop() {
         if (!isEmpty()) {
            return rack[top--];
         } else {
            return (T)returnType;
         }
      }

      /**
       * Returns true if the stack is full; otherwise, false
       */ 
      public bool isFull() {
         if (top >= size - 1)
            return true;
         else
            return false;
      }

      /**
       * Returns true if the stack empty; otherwise, false
       */ 
      public bool isEmpty() {
         if (top < 0)
            return true;
         else
            return false;
      }

      /**
       * Makes the stack empty
       */ 
      public void clear() { top = -1; }

      /**
       * Returns the top element index
       */ 
      public int getIndex() { return top; }
   }
}